# R program to read files of SAIPE data for CPS poverty rate equations
# User specifies the set of years to read in, the age group desired, 
# and whether to read files for related poverty rates (cpsxxp.txt) 
# or total poverty rates (cpsxxt.txt). Data are inserted into arrays 
# for the dependent variables (y), the regression matrices (X), and 
# the sampling std. deviations (3 versions: cpsse, gvfse, and fnlse. 
# See the readme file for discussion). If only one year is being read 
# in it may be as or more convenient to do this directly rather than 
# use this program.

# Program written by Bill Bell, 7/23/2015. Comments revised 1/16/2020.
# Corrected by Bill Bell, 10/25/2019. Assignment of cpsse set to the
#  correct column (cpssecol). It had previously been set to gvfsecol.


#--------------------------------------------------------------------
# Specify directory to read from (where data files are stored), years
# of data to read in, age group desired, and whether to read from 
# related or total poverty files. Note the format for years is two 
# digits as characters, which can range from 89 to 05, excluding 94. 
# Actually, numbers can be used for the years from 89 to 99.

  dir <- "directory path"     # Example: dir <- "c:/CPSeq-ts/"
  options(width=140,digits=5)

#  yrs  <- c(89:93,95:99,"00","01","02","03","04","05")   # read all years
  yrs  <- c("00","01","02","03","04","05")

  age  <- 2     # 1 ~ 0-4, 2 ~ 5-17, 3 ~ 18-64, 4 ~ 65+

  type <- "p"   # if type = "t", read in files on total poverty rates


#-------------------------------------------------------------------
# For each year the files contain 7 variables for each age group 
# across 7 columns in the following order:
#   CPS direct estimate (y)
#   three regression variables (X)
#   CPS estimate of sampling std. error (cpsse)
#   GVF estimate of sampling std. error (gvfse)
#   final sampling std. error after REML iterations (fnlse)
# These will be extracted into 6 column vectors and, for X, a matrix.
# Set the corresponding columns appropriate to the age group selected.

  ycol     <- (age - 1)*7 + 1    # Note X variables are in the next 3 columns
  cpssecol <- (age - 1)*7 + 5
  gvfsecol <- (age - 1)*7 + 6
  fnlsecol <- (age - 1)*7 + 7

  nxvars <- 3


#-----------------------------------
# Set up arrays for storing the data

  nyrs  <- length(yrs)
  y     <- matrix(0,nrow=51,ncol=nyrs)
  gvfse <- matrix(0,nrow=51,ncol=nyrs)
  cpsse <- matrix(0,nrow=51,ncol=nyrs)
  fnlse <- matrix(0,nrow=51,ncol=nyrs)
  X     <- array(0,dim=c(51,nxvars,nyrs))


#------------------------------------------------------------------
# Loop over the elements of yrs reading in the files and extracting 
# the columns desired.


  for (i in 1:nyrs) {
    yr <- yrs[i]
    tmpfile <- paste(dir,"CPS",yr,type,".txt",sep="")
    tmp     <- read.table(tmpfile,skip=1)

    y[,i]     <- tmp[,ycol]
    cpsse[,i] <- tmp[,cpssecol]
    gvfse[,i] <- tmp[,gvfsecol]
    fnlse[,i] <- tmp[,fnlsecol]

    for (j in 1:51)    {
    for (k in 1:nxvars)  {X[j,k,i] <- tmp[j,ycol + k]}
    }
  }


